@@ -1,4 +1,4 @@
-from pulsar import send
+from pulsar import send, coroutine_return
 
 from .jsonrpc import JSONRPC
 
@@ -7,7 +7,7 @@ __all__ = ['PulsarServerCommands']
 
 
 class PulsarServerCommands(JSONRPC):
-    '''Useful commands to add to your :class:`JSONRPC` handler.
+    '''Useful commands to add to your :class:`.JSONRPC` handler.
 
     It exposes the following functions:'''
     def rpc_ping(self, request):
@@ -25,11 +25,12 @@ class PulsarServerCommands(JSONRPC):
         information.
         '''
         info = yield send('arbiter', 'info')
-        yield self.extra_server_info(request, info)
+        info = yield self.extra_server_info(request, info)
+        coroutine_return(info)
 
     def rpc_functions_list(self, request):
         '''List of (method name, method document) pair of all method exposed
-        by this :class:`JSONRPC` handler.'''
+        by this :class:`.JSONRPC` handler.'''
         return list(self.listFunctions())
 
     def rpc_documentation(self, request):
