@@ -99,10 +99,9 @@ import socket
 from math import log
 from random import lognormvariate
 from functools import partial
-from asyncio import DatagramProtocol, Protocol
 
 import pulsar
-from pulsar import TcpServer, DatagramServer, Connection
+from pulsar import asyncio, TcpServer, DatagramServer, Connection
 from pulsar.utils.internet import (parse_address, SSLContext, WrapSocket,
                                    format_address)
 from pulsar.utils.config import pass_through
@@ -241,7 +240,7 @@ class SocketServer(pulsar.Application):
             ssl = SSLContext(keyfile=cfg.key_file, certfile=cfg.cert_file)
         address = parse_address(self.cfg.address)
         # First create the sockets
-        server = yield loop.create_server(Protocol, *address)
+        server = yield loop.create_server(asyncio.Protocol, *address)
         addresses = []
         sockets = []
         for sock in server.sockets:
@@ -339,7 +338,8 @@ class UdpSocketServer(SocketServer):
                                               'No address to bind to')
         address = parse_address(self.cfg.address)
         # First create the sockets
-        t, _ = yield loop.create_datagram_endpoint(DatagramProtocol, address)
+        t, _ = yield loop.create_datagram_endpoint(asyncio.DatagramProtocol,
+                                                   address)
         sock = t.get_extra_info('socket')
         assert loop.remove_reader(sock.fileno())
         monitor.sockets = [WrapTransport(t)]
